/*
 * natstat.c
 * Copyright (c) M. Ulikowski <elceef@itsec.pl>
 * This file is part of NATDet
*/

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define  MAX_NAT        32
#define  MAX_LOG_LINES  65535

static struct natdet_stats {
  u_char addr[16];
  u_int  prob,
         oscount;
} stats[MAX_NAT];


int main(int argc, char *argv[]) {
  u_char line[80], time[64], addr[16];
  u_int oscount, prob, i, k;
  FILE *log;

  if (argc < 2) {
    fprintf(stderr, "Usage: %s <logfile> [outfile]\n", argv[0]);
    exit(0);
  }

  if (!(log = fopen(argv[1], "r"))) {
    fprintf(stderr, "[!] Unable to open \"%s\"\n", argv[1]);
    exit(1);
  }

  for (i = 0; i < MAX_LOG_LINES; i++) {
    memset(line, '\0', sizeof(line));
    fgets(line, sizeof(line), log);

    if (feof(log))
      break;

    if (strstr(line, "NATDet ")) {
      strncpy(time, line, sizeof(time));
      time[strlen(time)-1] = '\0';
      continue;
    }

    if (line[1] != '*')
      continue;

    memcpy(line, line+32, 48);

    if (sscanf(line, "%[1234567890.] for %u system(s) [%u%%]\n",
        addr, &oscount, &prob) != 3)
      continue;

    for (k = 0; k < MAX_NAT; k++) {
      if (!strcmp(stats[k].addr, addr)) {
        stats[k].prob = (stats[k].prob + prob) / 2;
        if (oscount > stats[k].oscount)
          stats[k].oscount = oscount;
        break;
      }
      if (!strlen(stats[k].addr)) {
        strcpy(stats[k].addr, addr);
        stats[k].prob    = prob;
        stats[k].oscount = oscount;
        break;
      }
    }

  }

  if (argv[2]) {
    if (!freopen(argv[2], "a", stdout))
      printf("[!] Unable to open '%s'\n", argv[2]);
  }

  printf("%s\n", time);

  for (k = 0; k < MAX_NAT; k++) {
    if (stats[k].prob) {
      printf("%-15s NAT detected with %3u%% accuracy (for %u systems)\n",
             stats[k].addr, stats[k].prob, stats[k].oscount);
    }
  }

  fclose(log);

  return 0;
}

